ENV['RAILS_ENV'] ||= 'test'
require File.expand_path('../config/environment', __dir__)
require 'rspec/rails'
require 'rspec/collection_matchers'
require 'capybara-screenshot/rspec'
require 'capybara/rails'
require 'capybara/rspec'
require 'selenium-webdriver'
require 'database_cleaner'
require 'vcr'
require 'opencpu'
Dir[Rails.root.join('spec/support/**/*.rb')].sort.each { |f| require f }

# enable locally to see logs of js  errors.
# RSpec.configure do |config|
#   config.after(type: :feature) do
#     STDERR.puts page.driver.browser.manage.logs.get(:browser)
#   end
# end

if ENV['SELENIUM_HOST'].present? && ENV['TEST_APP_PORT'].present?
  # we live in docker-compose,
  # dependencies can't easily link back to main service by using service name as hostname,
  # so we find our ip.
  docker_ip = `hostname -i`.strip
  Capybara.app_host = "http://#{docker_ip}:#{ENV['TEST_APP_PORT']}"
  Capybara.server_host = '0.0.0.0'
  Capybara.server_port = ENV['TEST_APP_PORT']
  selenium_url = "http://#{ENV['SELENIUM_HOST']}:#{ENV['SELENIUM_PORT']}/wd/hub"

  VCR.configuration.ignore_hosts << ENV['SELENIUM_HOST']
end
Capybara.register_driver :selenium_chrome_headless do |app|
  options = Selenium::WebDriver::Chrome::Options.new
  options.add_argument('--headless')
  options.add_argument('--disable-gpu')
  options.add_argument('--no-sandbox')

  Capybara::Selenium::Driver.new app,
                                 browser: :chrome,
                                 url: selenium_url,
                                 options: options
end
Capybara.javascript_driver = :selenium_chrome_headless

RSpec.configure do |config|
  config.include RSpec::Repeat

  config.around :example, type: :feature do |example|
    repeat example, 5.times, verbose: true
  end

  config.expect_with :rspec do |expectations|
    expectations.include_chain_clauses_in_custom_matcher_descriptions = true
  end

  config.disable_monkey_patching!
  config.infer_spec_type_from_file_location!

  if config.files_to_run.one?
    config.default_formatter = 'doc'
  else
    SimpleCov.start 'rails'
  end

  config.order = :random
  config.expose_dsl_globally = true

  # Add focus: true to a context to run only that test
  config.filter_run focus: true
  config.run_all_when_everything_filtered = true

  save_screenshots_as_circleci_artifacts

  mock_all_calls_to_r

  Rails.application.routes.default_url_options[:host] = 'test_host'

  DatabaseCleaner.allow_remote_database_url = true

  config.before(:suite) do
    DatabaseCleaner.clean_with(:truncation)
  end

  config.before(:each) do
    DatabaseCleaner.strategy = :transaction
  end
end
